/////////////////////////////////////////////////////////////////////////////
// Name:        nviz3d.cpp
// Purpose:     NeReK OpenGL Visualization Code
// Author:      Chad Rempp
// Modified by: Chad Rempp
// Created:     06-01-03
// Copyright:   (c) Chad Rempp 2003
// Licence:     GNU Public Licence
/////////////////////////////////////////////////////////////////////////////

//TODO
/*
   The camera is situated at the origin and every thing rotates around it
   it should be the other way around.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation
#pragma interface
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#if !wxUSE_GLCANVAS
#error Please set wxUSE_GLCANVAS to 1 in setup.h.
#endif

#include "nviz3d/nviz3d.h"

// !!--TODO--!!
// Try to get rid of these.

GLfloat neuron_radius = 0.5;
int neuron_stacks = 10;
int neuron_slices = 10;

static float last_x = 0,
             last_y = 0;
static GLfloat camera_x = 0;
static GLfloat camera_y = 0;
static GLfloat camera_zoom = -6.0;
static GLfloat camera_rotate = 0.0;

GLfloat drag_scale_x = 0.05;
GLfloat drag_scale_y = 0.05;
GLfloat rotate_scale = 0.01;
GLfloat zoom_scale   = 0.05;

wxTextCtrl *txt_drag_scale_x;
wxTextCtrl *txt_drag_scale_y;

GLUquadricObj *gl_neuron_00 = gluNewQuadric();
GLUquadricObj *gl_neuron_01 = gluNewQuadric();
GLUquadricObj *gl_neuron_02 = gluNewQuadric();
GLUquadricObj *gl_neuron_03 = gluNewQuadric();
GLUquadricObj *gl_neuron_04 = gluNewQuadric();
GLUquadricObj *gl_neuron_05 = gluNewQuadric();

///////////////////////////////////////////////////////////////////////////////
//  NViz3DCanvas class Implementation
///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE(Nviz3DCanvas, wxGLCanvas)
    EVT_SIZE(Nviz3DCanvas::OnSize)
    EVT_PAINT(Nviz3DCanvas::OnPaint)
    EVT_ERASE_BACKGROUND(Nviz3DCanvas::OnEraseBackground)
    EVT_ENTER_WINDOW( Nviz3DCanvas::OnEnterWindow )
    EVT_MOUSE_EVENTS( Nviz3DCanvas::OnMouse )
END_EVENT_TABLE()

//////////////////////Nviz3DCanvas Constructor/////////////////////////////////
Nviz3DCanvas::Nviz3DCanvas(wxWindow *parent, wxWindowID id,
                           const wxPoint& pos, const wxSize& size,
                           long style, const wxString& name):
             wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}

////////////Nviz3DCanvas Destructor////////////////////////////////////////////
Nviz3DCanvas::~Nviz3DCanvas()
{
    // Nothing Here
}

////////////Nviz3DCanvas DrawNeuron Method/////////////////////////////////////
void Nviz3DCanvas::DrawNeuron(GLUquadricObj *obj, GLfloat x, GLfloat y, GLfloat z)
{
    glPushMatrix();
    glTranslatef(x, y, z);
    gluSphere(obj, neuron_radius, neuron_stacks, neuron_slices);
    glPopMatrix();
}

////////////Nviz3DCanvas InitGL Method/////////////////////////////////////////
void Nviz3DCanvas::InitGL()
{
    SetCurrent();

    // set viewing projection
    glMatrixMode(GL_PROJECTION);
    glFrustum(-1.0F, 1.0F, -1.0F, 1.0F, 1.0F, 100.0F);

    // position viewer
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslated(0.0, 0.0, -4.0);

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
}

////////////Nviz3DCanvas OnEnterWindow Method//////////////////////////////////
void Nviz3DCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

////////////Nviz3DCanvas OnEraseBackGround Method//////////////////////////////
void Nviz3DCanvas::OnEraseBackground(wxEraseEvent& event)
{
  // Do nothing, to avoid flashing.
}

////////////Nviz3DCanvas OnMouse Method////////////////////////////////////////
void Nviz3DCanvas::OnMouse(wxMouseEvent &event)
{
// I think all this dragging stuff is a way to waste a cycle to get a last_x or
// y before drawing. Still not working but good enough for now.
    static int dragging = 0;

    if (event.LeftIsDown())
    {
        if (!dragging)
        {
            dragging = 1;
        }
        else
        {
            camera_x += (event.GetX() - last_x)*(drag_scale_x);
            camera_y -= (event.GetY() - last_y)*(drag_scale_y);
            Refresh(FALSE);
        }
        last_x = event.GetX();
        last_y = event.GetY();
    }
    else if (event.RightIsDown())
    {
        if (!dragging)
        {
            dragging = 1;
        }
        else
        {
            camera_rotate += (event.GetX() - last_x)*(rotate_scale);
            camera_zoom  += (event.GetY() - last_y)*(zoom_scale);
            Refresh(FALSE);
        }
        last_x = event.GetX();
        last_y = event.GetY();
    }
    else
    {
        dragging = 0;
        camera_rotate = 0;
    }
}

////////////Nviz3DCanvas OnPaint Method////////////////////////////////////////
void Nviz3DCanvas::OnPaint( wxPaintEvent& event )
{
    Render();
}

////////////Nviz3DCanvas OnSize Method/////////////////////////////////////////
void Nviz3DCanvas::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);

    // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
    int w, h;
    GetClientSize(&w, &h);
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
        glViewport(0, 0, (GLint) w, (GLint) h);
    }
}

////////////Nviz3DCanvas Render Method/////////////////////////////////////////
void Nviz3DCanvas::Render()
{
    // This is apperantly a dummy to avoid an endless succession of paint
    // messages. OnPaint handlers must always create a wxPaintDC.
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();
    // init OpenGL once, but after SetCurrent
    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }

    // clear color and depth buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    glMatrixMode(GL_MODELVIEW);
    glTranslatef(camera_x, camera_y, camera_zoom);
    glMatrixMode(GL_PROJECTION);
    glRotatef(camera_rotate, 0.0, 1.0, 0.0);
    glMatrixMode(GL_MODELVIEW);

    DrawNeuron(gl_neuron_00 , -2.0, -4.0, 0.0);
    DrawNeuron(gl_neuron_01 , 2.0, -4.0, 0.0);
    DrawNeuron(gl_neuron_02 , -2.0, 0.0, 0.0);
    DrawNeuron(gl_neuron_03 , 2.0, .0, 0.0);
    DrawNeuron(gl_neuron_04 , -2.0, 4.0, 0.0);
    DrawNeuron(gl_neuron_05 , 2.0, 4.0, 0.0);

    glLineWidth(8);
    glBegin(GL_LINES);
       glVertex3f(-2.0, -4.0, 0.0);
       glVertex3f(-2.0, 0.0, 0.0);
    glEnd();

    glPopMatrix();

  glFlush();
  SwapBuffers();
}

///////////////////////////////////////////////////////////////////////////////
//  NVizFrame class Implementation
///////////////////////////////////////////////////////////////////////////////

BEGIN_EVENT_TABLE(NvizFrame, wxFrame)
    EVT_MENU(wxID_EXIT, NvizFrame::OnExit)
END_EVENT_TABLE()

////////////NvizFrame Constructor//////////////////////////////////////////////
NvizFrame::NvizFrame(wxDialog *parent, const wxString& title, const wxPoint& pos,
                     const wxSize& size, long style)
                     : wxFrame(parent, -1, title, pos, size, style)
{
    glc_nvizcanvas = new Nviz3DCanvas(this, -1, wxDefaultPosition, wxDefaultSize);
}

////////////NvizFrame OnExit Method////////////////////////////////////////////
void NvizFrame::OnExit(wxCommandEvent &event)
{
    Destroy();
}

